/**
 * Name: threaded_open_test.cpp
 * Desc: Multi-threaded attempt to open devices in an attempt to uncover
 *       synchronization issues.
**/

#include <stdio.h>

#include <boost/asio/io_service.hpp>
#include <boost/scope_exit.hpp>
#include <boost/thread.hpp>

#include <LabJackM.h>

#include "../LJM_Utilities.h"

// #include "util/Device.h"
// #include "util/DeviceList.h"
#include "util/SemaphoreToMutexAdapter.h"
#include "util/ThreadPool.h"
#include "util/Util.h"

static const int NUM_THREADS = 4;

void OpenCall(
	ThreadPool & threadpool,
	const char * devType,
	const char * connType,
	const char * identifier
);

void ListAllCall(
	ThreadPool & threadpool,
	const char * devType,
	const char * connType
);

int main(int argc, char * argv[]) {
	printf("Beginning open calls with %d threads\n", NUM_THREADS);
	ThreadPool threadpool(NUM_THREADS);

	threadpool.AddWorker(
		boost::bind(ListAllCall, _1, "LJM_dtANY", "LJM_ctETHERNET")
	);

	threadpool.AddWorker(
		boost::bind(OpenCall, _1, "LJM_dtANY", "LJM_ctETHERNET", "LJM_idANY")
	);

	threadpool.AddWorker(
		boost::bind(OpenCall, _1, "LJM_dtANY", "LJM_ctUSB", "LJM_idANY")
	);

	threadpool.AddWorker(
		boost::bind(OpenCall, _1, "LJM_dtANY", "LJM_ctWIFI", "LJM_idANY")
	);

	threadpool.Run();
	threadpool.JoinAll();
}

// void OpenAllCall(
// 	ThreadPool & threadpool,
// 	const char * devType,
// 	const char * connType,
// 	const char * identifier
// ) {
// 	printf("Internal_LJM_OpenAll(%s, %s, %s, ...)\n", devType, connType, identifier);
// }

void ListAllCall(
	ThreadPool & threadpool,
	const char * devType,
	const char * connType
) {
	UNUSED(threadpool);

	for (int cou = 0; cou < 1; cou++) {
		printf("LJM_ListAllS(%s, %s, ...)\n", devType, connType);
		int numFound;
		int aDeviceTypes[LJM_LIST_ALL_SIZE];
		int aConnectionTypes[LJM_LIST_ALL_SIZE];
		int aSerialNumbers[LJM_LIST_ALL_SIZE];
		int aIPAddresses[LJM_LIST_ALL_SIZE];

		int err = LJM_ListAllS(
			devType,
			connType,
			&numFound,
			aDeviceTypes,
			aConnectionTypes,
			aSerialNumbers,
			aIPAddresses
		);
		PrintErrorIfError(
			err,
			"LJM_ListAllS(%s, %s, ...)",
			devType,
			connType
		);

		printf("end LJM_ListAllS(%s, %s, ...):    %d\n", devType, connType, numFound);
	}
}

void OpenCall(
	ThreadPool & threadpool,
	const char * devType,
	const char * connType,
	const char * identifier
) {
	UNUSED(threadpool);

	for (int cou = 0; cou < 1; cou++) {
		printf("LJM_OpenS(%s, %s, %s, ...)\n", devType, connType, identifier);
		int handle = 0;
		int err = LJM_OpenS(devType, connType, identifier, &handle);
		PrintErrorIfError(
			err,
			"LJM_OpenS(%s, %s, %s, ...)",
			devType,
			connType,
			identifier
		);


		// TODO: Ras;dfkjads;fj
		LJM_Close(handle);


		printf("end LJM_OpenS(%s, %s, %s, ...)\n", devType, connType, identifier);
	}

}
